@@ -1,6 +1,7 @@
 #include "kanjipad.h"
 #include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 static void
 pad_area_free_stroke (GList *stroke)
@@ -38,6 +39,7 @@ pad_area_annotate_stroke (PadArea *area, GList *stroke, gint index)
       if (stroke)
 	{
 	  char buffer[16];
+	  PangoLayout *layout;
 	  int swidth, sheight;
 	  gint16 x, y;
 	  double r;
@@ -46,31 +48,34 @@ pad_area_annotate_stroke (PadArea *area, GList *stroke, gint index)
 	  double dl = sqrt(dx*dx+dy*dy);
 	  int sign = (dy <= dx) ? 1 : -1;
 	  GdkRectangle update_area;
-	  
+
 	  sprintf (buffer, "%d", index);
-	  swidth = gdk_string_width (area->widget->style->font, buffer);
-	  sheight = area->widget->style->font->ascent + area->widget->style->font->descent;
+	  layout = gtk_widget_create_pango_layout (area->widget, buffer);
+	  pango_layout_get_pixel_size (layout, &swidth, &sheight);
+
 	  r = sqrt(swidth*swidth + sheight*sheight);
 	  
 	  x = 0.5 + old->x + 0.5*r*dx/dl + sign * 0.5*r*dy/dl;
 	  y = 0.5 + old->y + 0.5*r*dy/dl - sign * 0.5*r*dx/dl;
 	  
 	  x -= swidth/2;
-	  y += sheight/2 - area->widget->style->font->descent;
+	  y -= sheight/2;
 
 	  update_area.x = x;
-	  update_area.y = y - area->widget->style->font->ascent;
+	  update_area.y = y;
 	  update_area.width = swidth;
 	  update_area.height = sheight;
 	  
 	  x = CLAMP (x, 0, area->widget->allocation.width - swidth);
-	  y = CLAMP (y, area->widget->style->font->ascent, 
-		     area->widget->allocation.height);
-	  
-	  gdk_draw_string (area->pixmap, area->widget->style->font,
-			   area->widget->style->black_gc, x, y, buffer);
+	  y = CLAMP (y, 0, area->widget->allocation.height - sheight);
 
-	  gtk_widget_draw (area->widget, &update_area);
+	  gdk_draw_layout (area->pixmap, 
+			   area->widget->style->black_gc,
+			   x, y, layout);
+
+	  g_object_unref (layout);
+
+	  gdk_window_invalidate_rect (area->widget->window, &update_area, FALSE);
 	}
     }
 }
@@ -115,7 +120,7 @@ pad_area_init (PadArea *area)
       index++;
     }
 
-  gtk_widget_draw (area->widget, NULL);
+  gtk_widget_queue_draw (area->widget);
   
 }
 
@@ -124,7 +129,7 @@ pad_area_configure_event (GtkWidget *w, GdkEventConfigure *event,
 			  PadArea *area)
 {
   if (area->pixmap)
-    gdk_pixmap_unref (area->pixmap);
+    g_object_unref (area->pixmap);
 
   area->pixmap = gdk_pixmap_new (w->window, event->width, event->height, -1);
 
@@ -139,11 +144,11 @@ pad_area_expose_event (GtkWidget *w, GdkEventExpose *event, PadArea *area)
   if (!area->pixmap)
     return 0;
 
-  gdk_draw_pixmap (w->window,
-		   w->style->fg_gc[GTK_STATE_NORMAL], area->pixmap,
-		   event->area.x, event->area.y,
-		   event->area.x, event->area.y,
-		   event->area.width, event->area.height);
+  gdk_draw_drawable (w->window,
+		     w->style->fg_gc[GTK_STATE_NORMAL], area->pixmap,
+		     event->area.x, event->area.y,
+		     event->area.x, event->area.y,
+		     event->area.width, event->area.height);
 
   return TRUE;
 }
@@ -173,7 +178,7 @@ pad_area_button_release_event (GtkWidget *w, GdkEventButton *event, PadArea *are
   area->curstroke = NULL;
   area->instroke = FALSE;
 
-/*  look_up_callback(w); */
+  pad_area_changed_callback (area);
 
   return TRUE;
 }
@@ -215,7 +220,7 @@ pad_area_motion_event (GtkWidget *w, GdkEventMotion *event, PadArea *area)
       rect.y = ymin = 1;
       rect.width  = xmax - xmin + 2;
       rect.height = ymax - ymin + 2;
-      gtk_widget_draw (w, &rect);
+      gdk_window_invalidate_rect (w->window, &rect, FALSE);
 
       p = g_new (GdkPoint, 1);
       p->x = x;
@@ -231,18 +236,18 @@ PadArea *pad_area_create ()
   PadArea *area = g_new (PadArea, 1);
   
   area->widget = gtk_drawing_area_new();
-  gtk_drawing_area_size (GTK_DRAWING_AREA(area->widget), 300, 300);
+  gtk_widget_set_size_request (area->widget, 100, 100);
 
-  gtk_signal_connect (GTK_OBJECT (area->widget), "configure_event",
-		      GTK_SIGNAL_FUNC (pad_area_configure_event), area);
-  gtk_signal_connect (GTK_OBJECT (area->widget), "expose_event",
-		      GTK_SIGNAL_FUNC (pad_area_expose_event), area);
-  gtk_signal_connect (GTK_OBJECT (area->widget), "button_press_event",
-		      GTK_SIGNAL_FUNC (pad_area_button_press_event), area);
-  gtk_signal_connect (GTK_OBJECT (area->widget), "button_release_event",
-		      GTK_SIGNAL_FUNC (pad_area_button_release_event), area);
-  gtk_signal_connect (GTK_OBJECT (area->widget), "motion_notify_event",
-		      GTK_SIGNAL_FUNC (pad_area_motion_event), area);
+  g_signal_connect (area->widget, "configure_event",
+		    G_CALLBACK (pad_area_configure_event), area);
+  g_signal_connect (area->widget, "expose_event",
+		    G_CALLBACK (pad_area_expose_event), area);
+  g_signal_connect (area->widget, "button_press_event",
+		    G_CALLBACK (pad_area_button_press_event), area);
+  g_signal_connect (area->widget, "button_release_event",
+		    G_CALLBACK (pad_area_button_release_event), area);
+  g_signal_connect (area->widget, "motion_notify_event",
+		    G_CALLBACK (pad_area_motion_event), area);
 
   gtk_widget_set_events (area->widget, 
 			 GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK 
@@ -286,6 +291,8 @@ void pad_area_clear (PadArea *area)
   area->curstroke = NULL;
 
   pad_area_init (area);
+
+  pad_area_changed_callback (area);  
 }
 
 void pad_area_set_annotate (PadArea *area, gint annotate)
